import pyedm
from logger import log
from tree_node import TreeNode
import bpy
import animation as anim
from mathutils import Matrix
from math_tools import IDENTITY_MATRIX

def build_bone_id(armature_name, bone_name):
    return f'{armature_name} : {bone_name}'

class BoneNode(TreeNode):
    def __init__(self, edm_parent: pyedm.Node, pbone: bpy.types.PoseBone, bone: bpy.types.Bone, armature: bpy.types.Armature) -> None:
        super().__init__()
        self.name = build_bone_id(armature.name, bone.name)
        self.armature = armature
        self.pbone = pbone
        self.bone = bone

        # it's rest pose.
        self.mat_inv = (self.bone.matrix_local).inverted()
        self.mat = self.pbone.matrix

        self.edm_bone = anim.extract_bone_animation(edm_parent, self, armature)

        #if pbone and pbone.parent:
        #    self.matrix_inv = self.pbone.parent.matrix.inverted()
        #else:
        #    self.matrix_inv = IDENTITY_MATRIX

def build_bones_tree(edm_parent, bones_list, bones, armature):
    out = [BoneNode(edm_parent, b, b.bone, armature) for b in bones_list]
    bones.update({(x.name, x) for x in out})

    for b in out:
        if b.bone.parent:
            p = bones[f'{armature.name} : {b.bone.parent.name}']
            #p.add_child(b)
            #p.edm_bone.addChild(b.edm_bone)

def export_armature(armature, parent, bones_dict):
    bones = armature.pose.bones
    if not bones:
        log.warning(f'Armature {armature.name} is empty')
        return
    
    #root = pyedm.Node(f'Armature {armature.name} Root')
    #root = pyedm.Transform(f'Armature {armature.name} Root', armature.matrix_local)
    #parent = parent.addChild(root)
    build_bones_tree(parent, bones, bones_dict, armature)
    return parent
