import sys
from typing import List, Tuple

from mathutils import Euler, Matrix, Quaternion, Vector

import pyedm

RIGHT_TRANSFORM_MATRIX = Matrix((
    (-1.0,  0.0,  0.0,  0.0),
    ( 0.0,  0.0, -1.0,  0.0),
    ( 0.0,  1.0,  0.0,  0.0),
    ( 0.0,  0.0,  0.0,  1.0)
))

ROOT_TRANSFORM_MATRIX = Matrix((
    ( 1.0,  0.0,  0.0,  0.0),
    ( 0.0,  0.0,  1.0,  0.0),
    ( 0.0, -1.0,  0.0,  0.0),
    ( 0.0,  0.0,  0.0,  1.0)
))

IDENTITY_MATRIX = Matrix((
    ( 1.0,  0.0,  0.0,  0.0),
    ( 0.0,  1.0,  0.0,  0.0),
    ( 0.0,  0.0,  1.0,  0.0),
    ( 0.0,  0.0,  0.0,  1.0)
))

def euler_to_quat(euler):
    eul = Euler(euler, 'XYZ')
    q = eul.to_quaternion()
    return q

def get_max(vecs : List[Vector]):
    max_x = max_y = max_z = sys.float_info.min
    for v in vecs:
        if v.x > max_x:
           max_x = v.x
        if v.y > max_y:
           max_y = v.y
        if v.z > max_z:
           max_z = v.z
    return (max_x, max_y, max_z)

def get_min(vecs : List[Vector]):
    min_x = min_y = min_z = sys.float_info.max
    for v in vecs:
        if v.x < min_x:
           min_x = v.x
        if v.y < min_y:
           min_y = v.y
        if v.z < min_z:
           min_z = v.z
    return (min_x, min_y, min_z)

def get_aa_bb(mat: Matrix) -> Tuple[Vector, Vector]:
    vecs: List[Vector] = []
    for i in (-1.0, 1.0):
        for j in (-1.0, 1.0):
            for k in (-1.0, 1.0):
                vec = mat @ Vector((i, j, k))
                vecs.append(vec)
        
    vec1 = Vector(get_min(vecs))
    vec2 = Vector(get_max(vecs))

    return (vec1, vec2)